==> ruby-xslt_0.8.1.tar.gz: ruby-xslt/parser.c <==
/**
 * Copyright (C) 2005 Gregoire Lejeune <gregoire.lejeune@free.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

#include "xslt.h"

extern int xmlLoadExtDtdDefaultValue;
char* parse( char *, int, char *, int, char ** );
VALUE object_to_string( VALUE );

/**
 * xOut = parser( char *xml, int iXmlType, char *xslt, int iXslType, char **pxParams );
 */
char* parse( char *xml, int iXmlType, char *xsl, int iXslType, char **pxParams ) {
  xsltStylesheetPtr cur = NULL;
  xmlDocPtr docxml = NULL, docxsl = NULL, res = NULL;
  int out;

/** Rem: For encoding */
xmlCharEncodingHandlerPtr encoder = NULL;
const xmlChar *encoding = NULL;

  xmlChar *doc_txt_ptr;
  int doc_txt_len;

  /** Act: Disable debug mode */
/*  xsltDebugSetDefaultTrace( XSLT_TRACE_NONE ); */
  xmlSubstituteEntitiesDefault( 1 );
  xmlLoadExtDtdDefaultValue = 1;

/** Act: Encoding support */
xmlInitCharEncodingHandlers( );

  /** Act: Parse XSL */
  if( iXslType == RUBY_XSLT_XSLSRC_TYPE_STR ) {
    docxsl = xmlParseMemory( xsl, strlen( xsl ) );
    // docxsl = xmlParseDoc( xmlCharStrdup( xsl ) );
    if( docxsl == NULL ) {
      xsltTransformError( NULL, NULL, NULL, "XML::XSLT: XSL parsing error\n" );
      return( NULL );
    }

    cur = xsltParseStylesheetDoc( docxsl );
    if( cur == NULL ) {
      xsltTransformError( NULL, NULL, NULL, "XML::XSLT: XSL Stylesheet parsing error\n" );
      return( NULL );
    }
  } else if( iXslType == RUBY_XSLT_XSLSRC_TYPE_FILE ) {
    cur = xsltParseStylesheetFile( xsl );
    if( cur == NULL ) {
      xsltTransformError( NULL, NULL, NULL, "XML::XSLT: XSL Stylesheet parsing error\n" );
      return( NULL );
    }
    
/** Act: Get encoding */
XSLT_GET_IMPORT_PTR( encoding, cur, encoding )
encoder = xmlFindCharEncodingHandler((char *)encoding);
  
if( encoding != NULL ) {
  encoder = xmlFindCharEncodingHandler((char *)encoding);
  if( (encoder != NULL) && (xmlStrEqual((const xmlChar *)encoder->name, (const xmlChar *) "UTF-8")) )
    encoder = NULL;
  }
}
  
  /** Act: Parse XML */
  if( iXmlType == RUBY_XSLT_XMLSRC_TYPE_STR ) {
    docxml = xmlParseMemory( xml, strlen( xml ) );
    if( docxml == NULL ) {
      xsltTransformError( NULL, NULL, NULL, "XML::XSLT: XML parsing error\n" );
      return( NULL );
    }
  } else if( iXmlType == RUBY_XSLT_XMLSRC_TYPE_FILE ) {
    docxml = xmlParseFile( xml );
    if( docxml == NULL ) {
      xsltTransformError( NULL, NULL, NULL, "XML::XSLT: XML parsing error\n" );
      return( NULL );
    }
  }
  
  res = xsltApplyStylesheet( cur, docxml, (const char**) pxParams );
  if( res == NULL ) {
    xsltTransformError( NULL, NULL, NULL, "XML::XSLT: Stylesheet apply error\n" );
    return( NULL );
  }
  
  out = xsltSaveResultToString( &doc_txt_ptr, &doc_txt_len, res, cur );

  xsltFreeStylesheet(cur);
  xmlFreeDoc(res);
  xmlFreeDoc(docxml);
//  if( iXslType == RUBY_XSLT_XSLSRC_TYPE_STR ) { xmlFreeDoc(docxsl); }

  xsltCleanupGlobals();
  xmlCleanupParser();
  return((char*)doc_txt_ptr);
}

/**
 * vOut = object_to_string( VALUE object );
 */
VALUE object_to_string( VALUE object ) {
  VALUE vOut = Qnil;
    
  switch( TYPE( object ) ) {
    case T_STRING:
      {
        if( isFile( STR2CSTR( object ) ) == 0 ) {
          vOut = object;
        } else {
          long iBufferLength;
          long iCpt;
          char *xBuffer;

          FILE* fStream = fopen( STR2CSTR( object ), "r" );
          if( fStream == NULL ) {
            return( Qnil );
          }

          fseek( fStream, 0L, 2 );
          iBufferLength = ftell( fStream );
          xBuffer = (char *)malloc( (int)iBufferLength + 1 );
          if( !xBuffer )
            rb_raise( rb_eNoMemError, "Memory allocation error" );

          xBuffer[iBufferLength] = 0;
          fseek( fStream, 0L, 0 );
          iCpt = fread( xBuffer, 1, (int)iBufferLength, fStream );
          
          if( iCpt != iBufferLength ) {
            free( (char *)xBuffer );
            rb_raise( rb_eSystemCallError, "Read file error" );
          }

          vOut = rb_str_new2( xBuffer );
          fclose( fStream );
        }
      }
      break;
    
    case T_DATA:
    case T_OBJECT: 
      {
        if( strcmp( getRubyObjectName( object ), "XML::Simple::Dom" ) == 0 ) {
          vOut = rb_funcall( object, rb_intern( "to_s" ), 0 );
        } else {
          rb_raise( rb_eSystemCallError, "Can't read XML from object %s", getRubyObjectName( object ) );
        }
      }
      break;
    
    default:
      rb_raise( rb_eArgError, "XML object #0x%x not supported", TYPE( object ) );
  }

  return( vOut );
}

==> ruby-xslt_0.8.2.tar.gz: ruby-xslt/parser.c <==
/**
 * Copyright (C) 2005 Gregoire Lejeune <gregoire.lejeune@free.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

#include "xslt.h"

extern int xmlLoadExtDtdDefaultValue;
char* parse( char *, int, char *, int, char ** );
VALUE object_to_string( VALUE );

/**
 * xOut = parser( char *xml, int iXmlType, char *xslt, int iXslType, char **pxParams );
 */
char* parse( char *xml, int iXmlType, char *xsl, int iXslType, char **pxParams ) {
  xsltStylesheetPtr cur = NULL;
  xmlDocPtr docxml = NULL, docxsl = NULL, res = NULL;
  int out;

/** Rem: For encoding */
xmlCharEncodingHandlerPtr encoder = NULL;
const xmlChar *encoding = NULL;

  xmlChar *doc_txt_ptr;
  int doc_txt_len;

  /** Act: Disable debug mode */
/*  xsltDebugSetDefaultTrace( XSLT_TRACE_NONE ); */
  xmlSubstituteEntitiesDefault( 1 );
  xmlLoadExtDtdDefaultValue = 1;

/** Act: Encoding support */
xmlInitCharEncodingHandlers( );

  /** Act: Parse XSL */
  if( iXslType == RUBY_XSLT_XSLSRC_TYPE_STR ) {
    docxsl = xmlParseMemory( xsl, strlen( xsl ) );
    // docxsl = xmlParseDoc( xmlCharStrdup( xsl ) );
    if( docxsl == NULL ) {
      rb_raise( rb_eRuntimeError, "XSL parsing error" );
      return( NULL );
    }

    cur = xsltParseStylesheetDoc( docxsl );
    if( cur == NULL ) {
      rb_raise( rb_eRuntimeError, "XSL Stylesheet parsing error" );
      return( NULL );
    }
  } else if( iXslType == RUBY_XSLT_XSLSRC_TYPE_FILE ) {
    cur = xsltParseStylesheetFile( xsl );
    if( cur == NULL ) {
      rb_raise( rb_eRuntimeError, "XSL Stylesheet parsing error" );
      return( NULL );
    }
    
/** Act: Get encoding */
XSLT_GET_IMPORT_PTR( encoding, cur, encoding )
encoder = xmlFindCharEncodingHandler((char *)encoding);
  
if( encoding != NULL ) {
  encoder = xmlFindCharEncodingHandler((char *)encoding);
  if( (encoder != NULL) && (xmlStrEqual((const xmlChar *)encoder->name, (const xmlChar *) "UTF-8")) )
    encoder = NULL;
  }
}
  
  /** Act: Parse XML */
  if( iXmlType == RUBY_XSLT_XMLSRC_TYPE_STR ) {
    docxml = xmlParseMemory( xml, strlen( xml ) );
    if( docxml == NULL ) {
      rb_raise( rb_eRuntimeError, "XML parsing error" );
      return( NULL );
    }
  } else if( iXmlType == RUBY_XSLT_XMLSRC_TYPE_FILE ) {
    docxml = xmlParseFile( xml );
    if( docxml == NULL ) {
      rb_raise( rb_eRuntimeError, "XML parsing error" );
      return( NULL );
    }
  }
  
  res = xsltApplyStylesheet( cur, docxml, (const char**) pxParams );
  if( res == NULL ) {
    rb_raise( rb_eRuntimeError, "Stylesheet transformation error" );
    return( NULL );
  }
  
  out = xsltSaveResultToString( &doc_txt_ptr, &doc_txt_len, res, cur );

  xsltFreeStylesheet(cur);
  xmlFreeDoc(res);
  xmlFreeDoc(docxml);
  // if( iXslType == RUBY_XSLT_XSLSRC_TYPE_STR ) { xmlFreeDoc(docxsl); }

  /* if this gets called, then all of the registered functions get dumped. 
   * not sure what the alternative is.
   * this needs to get called when we're done with the module. */
  // xsltCleanupGlobals(); 
  xmlCleanupParser();
  return((char*)doc_txt_ptr);
}

/**
 * vOut = object_to_string( VALUE object );
 */
VALUE object_to_string( VALUE object ) {
  VALUE vOut = Qnil;
    
  switch( TYPE( object ) ) {
    case T_STRING:
      {
        if( isFile( STR2CSTR( object ) ) == 0 ) {
          vOut = object;
        } else {
          long iBufferLength;
          long iCpt;
          char *xBuffer;

          FILE* fStream = fopen( STR2CSTR( object ), "r" );
          if( fStream == NULL ) {
            return( Qnil );
          }

          fseek( fStream, 0L, 2 );
          iBufferLength = ftell( fStream );
          xBuffer = (char *)malloc( (int)iBufferLength + 1 );
          if( !xBuffer )
            rb_raise( rb_eNoMemError, "Memory allocation error" );

          xBuffer[iBufferLength] = 0;
          fseek( fStream, 0L, 0 );
          iCpt = fread( xBuffer, 1, (int)iBufferLength, fStream );
          
          if( iCpt != iBufferLength ) {
            free( (char *)xBuffer );
            rb_raise( rb_eSystemCallError, "Read file error" );
          }

          vOut = rb_str_new2( xBuffer );

          fclose( fStream );
        }
      }
      break;
    
    case T_DATA:
    case T_OBJECT: 
      {
        if( strcmp( getRubyObjectName( object ), "XML::Simple::Dom" ) == 0 ) {
          vOut = rb_funcall( object, rb_intern( "to_s" ), 0 );
        } else {
          rb_raise( rb_eSystemCallError, "Can't read XML from object %s", getRubyObjectName( object ) );
        }
      }
      break;
    
    default:
      rb_raise( rb_eArgError, "XML object #0x%x not supported", TYPE( object ) );
  }

  return( vOut );
}
